# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/framework/optimized_function_graph.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.core.framework import graph_pb2 as tensorflow_dot_core_dot_framework_dot_graph__pb2
from tensorflow.core.framework import types_pb2 as tensorflow_dot_core_dot_framework_dot_types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n8tensorflow/core/framework/optimized_function_graph.proto\x12\ntensorflow\x1a%tensorflow/core/framework/graph.proto\x1a%tensorflow/core/framework/types.proto\"\xb4\x02\n\x16OptimizedFunctionGraph\x12\x0c\n\x04name\x18\x01 \x01(\t\x12,\n\x0e\x66unction_graph\x18\x02 \x01(\x0b\x32\x14.tensorflow.GraphDef\x12^\n\x18node_name_to_control_ret\x18\x03 \x03(\x0b\x32<.tensorflow.OptimizedFunctionGraph.NodeNameToControlRetEntry\x12\'\n\tret_types\x18\x04 \x03(\x0e\x32\x14.tensorflow.DataType\x12\x18\n\x10num_return_nodes\x18\x05 \x01(\r\x1a;\n\x19NodeNameToControlRetEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.framework.optimized_function_graph_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _OPTIMIZEDFUNCTIONGRAPH_NODENAMETOCONTROLRETENTRY._options = None
  _OPTIMIZEDFUNCTIONGRAPH_NODENAMETOCONTROLRETENTRY._serialized_options = b'8\001'
  _OPTIMIZEDFUNCTIONGRAPH._serialized_start=151
  _OPTIMIZEDFUNCTIONGRAPH._serialized_end=459
  _OPTIMIZEDFUNCTIONGRAPH_NODENAMETOCONTROLRETENTRY._serialized_start=400
  _OPTIMIZEDFUNCTIONGRAPH_NODENAMETOCONTROLRETENTRY._serialized_end=459
# @@protoc_insertion_point(module_scope)
